@@ -1,5 +1,3 @@
-from __future__ import absolute_import, unicode_literals
-
 SPARSE_TTF_MASTER_TABLES = frozenset(
     ["glyf", "head", "hmtx", "loca", "maxp", "post", "vmtx"]
 )
@@ -16,6 +14,7 @@ FEATURE_WRITERS_KEY = UFO2FT_PREFIX + "featureWriters"
 
 USE_PRODUCTION_NAMES = UFO2FT_PREFIX + "useProductionNames"
 GLYPHS_DONT_USE_PRODUCTION_NAMES = GLYPHS_PREFIX + "Don't use Production Names"
+KEEP_GLYPH_NAMES = UFO2FT_PREFIX + "keepGlyphNames"
 
 COLOR_LAYERS_KEY = UFO2FT_PREFIX + "colorLayers"
 COLOR_PALETTES_KEY = UFO2FT_PREFIX + "colorPalettes"
