@@ -1,46 +1,44 @@
-# -*- coding: utf-8 -*-
-from __future__ import print_function, division, absolute_import, unicode_literals
-from fontTools.misc.py23 import byteord, tounicode, round, unichr, BytesIO
-
 import logging
 import math
 from collections import Counter, namedtuple
+from io import BytesIO
 from types import SimpleNamespace
 
-from fontTools.ttLib import TTFont, newTable
 from fontTools.cffLib import (
-    TopDictIndex,
-    TopDict,
     CharStrings,
-    SubrsIndex,
     GlobalSubrsIndex,
-    PrivateDict,
     IndexedStrings,
+    PrivateDict,
+    SubrsIndex,
+    TopDict,
+    TopDictIndex,
 )
-from fontTools.pens.boundsPen import ControlBoundsPen
-from fontTools.pens.t2CharStringPen import T2CharStringPen
-from fontTools.pens.ttGlyphPen import TTGlyphPen
-from fontTools.ttLib.tables.O_S_2f_2 import Panose
-from fontTools.ttLib.tables._h_e_a_d import mac_epoch_diff
-from fontTools.ttLib.tables._g_l_y_f import Glyph, USE_MY_METRICS
 from fontTools.misc.arrayTools import unionRect
 from fontTools.misc.fixedTools import otRound
+from fontTools.pens.boundsPen import ControlBoundsPen
+from fontTools.pens.reverseContourPen import ReverseContourPen
+from fontTools.pens.t2CharStringPen import T2CharStringPen
+from fontTools.pens.ttGlyphPen import TTGlyphPen
+from fontTools.ttLib import TTFont, newTable
+from fontTools.ttLib.tables._g_l_y_f import USE_MY_METRICS, Glyph
+from fontTools.ttLib.tables._h_e_a_d import mac_epoch_diff
+from fontTools.ttLib.tables.O_S_2f_2 import Panose
 
+from ufo2ft.constants import COLOR_LAYERS_KEY, COLOR_PALETTES_KEY
 from ufo2ft.errors import InvalidFontData
 from ufo2ft.fontInfoData import (
-    getAttrWithFallback,
-    dateStringToTimeValue,
     dateStringForNow,
+    dateStringToTimeValue,
+    getAttrWithFallback,
     intListToNum,
     normalizeStringForPostscript,
 )
 from ufo2ft.util import (
+    _copyGlyph,
+    calcCodePageRanges,
     makeOfficialGlyphOrder,
     makeUnicodeToGlyphNameMapping,
-    calcCodePageRanges,
 )
-from ufo2ft.constants import COLOR_LAYERS_KEY, COLOR_PALETTES_KEY
-
 
 logger = logging.getLogger(__name__)
 
@@ -66,21 +64,40 @@ def _getVerticalOrigin(font, glyph):
     return otRound(verticalOrigin)
 
 
-class BaseOutlineCompiler(object):
+class BaseOutlineCompiler:
     """Create a feature-less outline binary."""
 
     sfntVersion = None
     tables = frozenset(
-        ["head", "hmtx", "hhea", "name", "maxp", "cmap", "OS/2", "post", "vmtx", "vhea",
-         "COLR", "CPAL"]
+        [
+            "head",
+            "hmtx",
+            "hhea",
+            "name",
+            "maxp",
+            "cmap",
+            "OS/2",
+            "post",
+            "vmtx",
+            "vhea",
+            "COLR",
+            "CPAL",
+        ]
     )
 
-    def __init__(self, font, glyphSet=None, glyphOrder=None, tables=None):
+    def __init__(
+        self,
+        font,
+        glyphSet=None,
+        glyphOrder=None,
+        tables=None,
+        notdefGlyph=None,
+    ):
         self.ufo = font
         # use the previously filtered glyphSet, if any
         if glyphSet is None:
             glyphSet = {g.name: g for g in font}
-        self.makeMissingRequiredGlyphs(font, glyphSet)
+        self.makeMissingRequiredGlyphs(font, glyphSet, self.sfntVersion, notdefGlyph)
         self.allGlyphs = glyphSet
         # store the glyph order
         if glyphOrder is None:
@@ -111,8 +128,9 @@ class BaseOutlineCompiler(object):
             getAttrWithFallback(self.ufo.info, metric) is not None
             for metric in vertical_metrics
         )
-        self.colorLayers = COLOR_LAYERS_KEY in self.ufo.lib and \
-                           COLOR_PALETTES_KEY in self.ufo.lib
+        self.colorLayers = (
+            COLOR_LAYERS_KEY in self.ufo.lib and COLOR_PALETTES_KEY in self.ufo.lib
+        )
 
         # write the glyph order
         self.otf.setGlyphOrder(self.glyphOrder)
@@ -179,7 +197,7 @@ class BaseOutlineCompiler(object):
         in a different way if desired.
         """
         fontBox = None
-        for glyphName, glyphBox in self.glyphBoundingBoxes.items():
+        for glyphBox in self.glyphBoundingBoxes.values():
             if glyphBox is None:
                 continue
             if fontBox is None:
@@ -207,7 +225,7 @@ class BaseOutlineCompiler(object):
         return makeUnicodeToGlyphNameMapping(self.allGlyphs, self.glyphOrder)
 
     @staticmethod
-    def makeMissingRequiredGlyphs(font, glyphSet):
+    def makeMissingRequiredGlyphs(font, glyphSet, sfntVersion, notdefGlyph=None):
         """
         Add .notdef to the glyph set if it is not present.
 
@@ -218,17 +236,24 @@ class BaseOutlineCompiler(object):
         if ".notdef" in glyphSet:
             return
 
-        unitsPerEm = otRound(getAttrWithFallback(font.info, "unitsPerEm"))
-        ascender = otRound(getAttrWithFallback(font.info, "ascender"))
-        descender = otRound(getAttrWithFallback(font.info, "descender"))
-        defaultWidth = otRound(unitsPerEm * 0.5)
-        glyphSet[".notdef"] = StubGlyph(
-            name=".notdef",
-            width=defaultWidth,
-            unitsPerEm=unitsPerEm,
-            ascender=ascender,
-            descender=descender,
-        )
+        reverseContour = sfntVersion == "\000\001\000\000"
+        if notdefGlyph:
+            notdefGlyph = _copyGlyph(notdefGlyph, reverseContour=reverseContour)
+        else:
+            unitsPerEm = otRound(getAttrWithFallback(font.info, "unitsPerEm"))
+            ascender = otRound(getAttrWithFallback(font.info, "ascender"))
+            descender = otRound(getAttrWithFallback(font.info, "descender"))
+            defaultWidth = otRound(unitsPerEm * 0.5)
+            notdefGlyph = StubGlyph(
+                name=".notdef",
+                width=defaultWidth,
+                unitsPerEm=unitsPerEm,
+                ascender=ascender,
+                descender=descender,
+                reverseContour=reverseContour,
+            )
+
+        glyphSet[".notdef"] = notdefGlyph
 
     def makeOfficialGlyphOrder(self, glyphOrder):
         """
@@ -354,7 +379,7 @@ class BaseOutlineCompiler(object):
             # ascii-encoded bytes when it can. On the other hand, fontTools's
             # name table `setName` method wants unicode strings, so we must
             # decode them first
-            nameVal = tounicode(nameRecord["string"], encoding="ascii")
+            nameVal = nameRecord["string"]
             name.setName(nameVal, nameId, platformId, platEncId, langId)
 
         # Build name records
@@ -366,7 +391,7 @@ class BaseOutlineCompiler(object):
         preferredSubfamilyName = getAttrWithFallback(
             font.info, "openTypeNamePreferredSubfamilyName"
         )
-        fullName = "%s %s" % (preferredFamilyName, preferredSubfamilyName)
+        fullName = f"{preferredFamilyName} {preferredSubfamilyName}"
 
         nameVals = {
             0: getAttrWithFallback(font.info, "copyright"),
@@ -386,6 +411,10 @@ class BaseOutlineCompiler(object):
             14: getAttrWithFallback(font.info, "openTypeNameLicenseURL"),
             16: preferredFamilyName,
             17: preferredSubfamilyName,
+            18: getAttrWithFallback(font.info, "openTypeNameCompatibleFullName"),
+            19: getAttrWithFallback(font.info, "openTypeNameSampleText"),
+            21: getAttrWithFallback(font.info, "openTypeNameWWSFamilyName"),
+            22: getAttrWithFallback(font.info, "openTypeNameWWSSubfamilyName"),
         }
 
         # don't add typographic names if they are the same as the legacy ones
@@ -401,7 +430,6 @@ class BaseOutlineCompiler(object):
             nameVal = nameVals[nameId]
             if not nameVal:
                 continue
-            nameVal = tounicode(nameVal, encoding="ascii")
             platformId = 3
             platEncId = 10 if _isNonBMP(nameVal) else 1
             langId = 0x409
@@ -433,13 +461,11 @@ class BaseOutlineCompiler(object):
 
         from fontTools.ttLib.tables._c_m_a_p import cmap_format_4
 
-        nonBMP = dict(
-            (k, v) for k, v in self.unicodeToGlyphNameMapping.items() if k > 65535
-        )
+        nonBMP = {k: v for k, v in self.unicodeToGlyphNameMapping.items() if k > 65535}
         if nonBMP:
-            mapping = dict(
-                (k, v) for k, v in self.unicodeToGlyphNameMapping.items() if k <= 65535
-            )
+            mapping = {
+                k: v for k, v in self.unicodeToGlyphNameMapping.items() if k <= 65535
+            }
         else:
             mapping = dict(self.unicodeToGlyphNameMapping)
         # mac
@@ -599,11 +625,8 @@ class BaseOutlineCompiler(object):
         os2.ulCodePageRange2 = intListToNum(codepageRanges, 32, 32)
 
         # vendor id
-        os2.achVendID = tounicode(
-            getAttrWithFallback(font.info, "openTypeOS2VendorID"),
-            encoding="ascii",
-            errors="ignore",
-        )
+        os2.achVendID = getAttrWithFallback(font.info, "openTypeOS2VendorID")
+
         # vertical metrics
         os2.sxHeight = otRound(getAttrWithFallback(font.info, "xHeight"))
         os2.sCapHeight = otRound(getAttrWithFallback(font.info, "capHeight"))
@@ -834,7 +857,10 @@ class BaseOutlineCompiler(object):
         vorg.defaultVertOriginY = vorg_count.most_common(1)[0][0]
         if len(vorg_count) > 1:
             for glyphName, glyph in self.allGlyphs.items():
-                vorg.VOriginRecords[glyphName] = _getVerticalOrigin(self.otf, glyph)
+                vertOriginY = _getVerticalOrigin(self.otf, glyph)
+                if vertOriginY == vorg.defaultVertOriginY:
+                    continue
+                vorg.VOriginRecords[glyphName] = vertOriginY
         vorg.numVertOriginYMetrics = len(vorg.VOriginRecords)
 
     def setupTable_vhea(self):
@@ -893,7 +919,9 @@ class BaseOutlineCompiler(object):
         from fontTools.colorLib.builder import buildCOLR
 
         layerInfo = self.ufo.lib[COLOR_LAYERS_KEY]
-        self.otf["COLR"] = buildCOLR(layerInfo)
+        glyphMap = self.otf.getReverseGlyphMap()
+        if layerInfo:
+            self.otf["COLR"] = buildCOLR(layerInfo, glyphMap=glyphMap)
 
     def setupTable_CPAL(self):
         """
@@ -936,12 +964,8 @@ class BaseOutlineCompiler(object):
         in a different way if desired.
         """
         import os
-        import re
 
         prefix = "com.github.fonttools.ttx"
-        sfntVersionRE = re.compile(
-            '(^<ttFont\s+)(sfntVersion=".*"\s+)(.*>$)', flags=re.MULTILINE
-        )
         if not hasattr(self.ufo, "data"):
             return
         if not self.ufo.data.fileNames:
@@ -950,11 +974,13 @@ class BaseOutlineCompiler(object):
             foldername, filename = os.path.split(path)
             if foldername == prefix and filename.endswith(".ttx"):
                 ttx = self.ufo.data[path].decode("utf-8")
-                # strip 'sfntVersion' attribute from ttFont element,
-                # if present, to avoid overwriting the current value
-                ttx = sfntVersionRE.sub(r"\1\3", ttx)
                 fp = BytesIO(ttx.encode("utf-8"))
-                self.otf.importXML(fp)
+                # Preserve the original SFNT version when loading a TTX dump.
+                sfntVersion = self.otf.sfntVersion
+                try:
+                    self.otf.importXML(fp)
+                finally:
+                    self.otf.sfntVersion = sfntVersion
 
 
 class OutlineOTFCompiler(BaseOutlineCompiler):
@@ -969,6 +995,7 @@ class OutlineOTFCompiler(BaseOutlineCompiler):
         glyphSet=None,
         glyphOrder=None,
         tables=None,
+        notdefGlyph=None,
         roundTolerance=None,
         optimizeCFF=True,
     ):
@@ -977,8 +1004,12 @@ class OutlineOTFCompiler(BaseOutlineCompiler):
         else:
             # round all coordinates to integers by default
             self.roundTolerance = 0.5
-        super(OutlineOTFCompiler, self).__init__(
-            font, glyphSet=glyphSet, glyphOrder=glyphOrder, tables=tables
+        super().__init__(
+            font,
+            glyphSet=glyphSet,
+            glyphOrder=glyphOrder,
+            tables=tables,
+            notdefGlyph=notdefGlyph,
         )
         self.optimizeCFF = optimizeCFF
         self._defaultAndNominalWidths = None
@@ -1262,8 +1293,7 @@ class OutlineOTFCompiler(BaseOutlineCompiler):
 
 
 class OutlineTTFCompiler(BaseOutlineCompiler):
-    """Compile a .ttf font with TrueType outlines.
-    """
+    """Compile a .ttf font with TrueType outlines."""
 
     sfntVersion = "\000\001\000\000"
     tables = BaseOutlineCompiler.tables | {"loca", "gasp", "glyf"}
@@ -1324,7 +1354,7 @@ class OutlineTTFCompiler(BaseOutlineCompiler):
 
     def setupTable_post(self):
         """Make a format 2 post table with the compiler's glyph order."""
-        super(OutlineTTFCompiler, self).setupTable_post()
+        super().setupTable_post()
         if "post" not in self.otf:
             return
 
@@ -1359,7 +1389,7 @@ class OutlineTTFCompiler(BaseOutlineCompiler):
 
     @staticmethod
     def autoUseMyMetrics(ttGlyph, glyphName, hmtx):
-        """ Set the "USE_MY_METRICS" flag on the first component having the
+        """Set the "USE_MY_METRICS" flag on the first component having the
         same advance width as the composite glyph, no transform and no
         horizontal shift (but allow it to shift vertically).
         This forces the composite glyph to use the possibly hinted horizontal
@@ -1382,28 +1412,38 @@ class OutlineTTFCompiler(BaseOutlineCompiler):
                     break
 
 
-class StubGlyph(object):
+class StubGlyph:
 
     """
     This object will be used to create missing glyphs
     (specifically .notdef) in the provided UFO.
     """
 
-    def __init__(self, name, width, unitsPerEm, ascender, descender, unicodes=[]):
+    def __init__(
+        self,
+        name,
+        width,
+        unitsPerEm,
+        ascender,
+        descender,
+        unicodes=None,
+        reverseContour=False,
+    ):
         self.name = name
         self.width = width
         self.unitsPerEm = unitsPerEm
         self.ascender = ascender
         self.descender = descender
-        self.unicodes = unicodes
+        self.unicodes = unicodes if unicodes is not None else []
         self.components = []
         self.anchors = []
-        if unicodes:
-            self.unicode = unicodes[0]
+        if self.unicodes:
+            self.unicode = self.unicodes[0]
         else:
             self.unicode = None
         if name == ".notdef":
             self.draw = self._drawDefaultNotdef
+        self.reverseContour = reverseContour
 
     def __len__(self):
         if self.name == ".notdef":
@@ -1418,6 +1458,10 @@ class StubGlyph(object):
         pass
 
     def _drawDefaultNotdef(self, pen):
+        # Draw contour in PostScript direction (counter-clockwise) by default. Reverse
+        # for TrueType.
+        if self.reverseContour:
+            pen = ReverseContourPen(pen)
         width = otRound(self.unitsPerEm * 0.5)
         stroke = otRound(self.unitsPerEm * 0.05)
         ascender = self.ascender
