@@ -1,12 +1,11 @@
-from __future__ import print_function, division, absolute_import, unicode_literals
-
-from textwrap import dedent
 import logging
+from textwrap import dedent
+
+import pytest
 
 from ufo2ft.featureCompiler import parseLayoutFeatures
 from ufo2ft.featureWriters import KernFeatureWriter, ast
 
-import pytest
 from . import FeatureWriterTest
 
 
@@ -226,7 +225,7 @@ class KernFeatureWriterTest(FeatureWriterTest):
         assert str(feaFile) == existing
 
     def test_arabic_numerals(self, FontClass):
-        """ Test that arabic numerals (with bidi type AN) are kerned LTR.
+        """Test that arabic numerals (with bidi type AN) are kerned LTR.
         https://github.com/googlei18n/ufo2ft/issues/198
         https://github.com/googlei18n/ufo2ft/pull/200
         """
@@ -294,7 +293,6 @@ class KernFeatureWriterTest(FeatureWriterTest):
         # feature file, and those generated by the feature writer
         font.features.text = "@kern1.A = [E F];"
 
-        writer = KernFeatureWriter()
         feaFile = parseLayoutFeatures(font)
         side1Classes, side2Classes = KernFeatureWriter.getKerningClasses(font, feaFile)
 
@@ -530,7 +528,8 @@ class KernFeatureWriterTest(FeatureWriterTest):
                 language URD;
             } fina;
 
-            @Bases = [A V Aacute alef-ar reh-ar zain-ar lam-ar alef-ar.isol lam-ar.init reh-ar.fina];
+            @Bases = [A V Aacute alef-ar reh-ar zain-ar lam-ar
+                      alef-ar.isol lam-ar.init reh-ar.fina];
             @Marks = [acutecomb fatha-ar];
             table GDEF {
                 GlyphClassDef @Bases, [], @Marks, ;
