@@ -1,14 +1,14 @@
-from __future__ import print_function, division, absolute_import, unicode_literals
-
 import os
 import random
 import time
+
+import pytest
+
 from ufo2ft.fontInfoData import (
+    dateStringToTimeValue,
     getAttrWithFallback,
     normalizeStringForPostscript,
-    dateStringToTimeValue,
 )
-import pytest
 
 
 @pytest.fixture
@@ -25,7 +25,7 @@ def info(InfoClass):
     return self
 
 
-class GetAttrWithFallbackTest(object):
+class GetAttrWithFallbackTest:
     @pytest.mark.parametrize(
         "infoDict,expected",
         [
@@ -39,7 +39,6 @@ class GetAttrWithFallbackTest(object):
                     "styleMapStyleName": "regular",
                     "openTypeNamePreferredFamilyName": "Family Name",
                     "openTypeNamePreferredSubfamilyName": "Style Name",
-                    "openTypeNameCompatibleFullName": "Family Name Style Name",
                 },
             ),
             # no styleMapStyleName
@@ -50,7 +49,6 @@ class GetAttrWithFallbackTest(object):
                     "styleMapStyleName": "regular",
                     "openTypeNamePreferredFamilyName": "Family Name",
                     "openTypeNamePreferredSubfamilyName": "Style Name",
-                    "openTypeNameCompatibleFullName": "Style Map Family Name",
                 },
             ),
             # no styleMapFamilyName, no styleMapStyleName but styleName="Regular"
@@ -63,7 +61,6 @@ class GetAttrWithFallbackTest(object):
                     "styleMapStyleName": "regular",
                     "openTypeNamePreferredFamilyName": "Family Name",
                     "openTypeNamePreferredSubfamilyName": "Regular",
-                    "openTypeNameCompatibleFullName": "Family Name",
                 },
             ),
             # no styleMapFamilyName but styleName="Regular"
@@ -74,7 +71,6 @@ class GetAttrWithFallbackTest(object):
                     "styleMapStyleName": "regular",
                     "openTypeNamePreferredFamilyName": "Family Name",
                     "openTypeNamePreferredSubfamilyName": "Regular",
-                    "openTypeNameCompatibleFullName": "Family Name",
                 },
             ),
             # no styleMapStyleName but styleName="Regular"
@@ -85,7 +81,6 @@ class GetAttrWithFallbackTest(object):
                     "styleMapStyleName": "regular",
                     "openTypeNamePreferredFamilyName": "Family Name",
                     "openTypeNamePreferredSubfamilyName": "Regular",
-                    "openTypeNameCompatibleFullName": "Style Map Family Name",
                 },
             ),
             # no styleMapFamilyName, no styleMapStyleName but styleName="Bold"
@@ -98,7 +93,6 @@ class GetAttrWithFallbackTest(object):
                     "styleMapStyleName": "bold",
                     "openTypeNamePreferredFamilyName": "Family Name",
                     "openTypeNamePreferredSubfamilyName": "Bold",
-                    "openTypeNameCompatibleFullName": "Family Name Bold",
                 },
             ),
         ],
@@ -151,13 +145,13 @@ class GetAttrWithFallbackTest(object):
         assert getAttrWithFallback(info, "openTypeHheaCaretSlopeRun") == -213
 
         info.openTypeHheaCaretSlopeRise = 2048
-        assert getattr(info, "openTypeHheaCaretSlopeRun") is None
+        assert info.openTypeHheaCaretSlopeRun is None
         assert getAttrWithFallback(info, "openTypeHheaCaretSlopeRise") == 2048
         assert getAttrWithFallback(info, "openTypeHheaCaretSlopeRun") == -435
 
         info.openTypeHheaCaretSlopeRise = None
         info.openTypeHheaCaretSlopeRun = 200
-        assert getattr(info, "openTypeHheaCaretSlopeRise") is None
+        assert info.openTypeHheaCaretSlopeRise is None
         assert getAttrWithFallback(info, "openTypeHheaCaretSlopeRise") == -941
         assert getAttrWithFallback(info, "openTypeHheaCaretSlopeRun") == 200
 
@@ -192,7 +186,7 @@ class GetAttrWithFallbackTest(object):
         assert getAttrWithFallback(info, "descender") == -410
 
 
-class PostscriptBlueScaleFallbackTest(object):
+class PostscriptBlueScaleFallbackTest:
     def test_without_blue_zones(self, info):
         postscriptBlueScale = getAttrWithFallback(info, "postscriptBlueScale")
         assert postscriptBlueScale == 0.039625
@@ -219,7 +213,7 @@ class PostscriptBlueScaleFallbackTest(object):
         assert postscriptBlueScale == 0.0375
 
 
-class NormalizeStringForPostscriptTest(object):
+class NormalizeStringForPostscriptTest:
     def test_no_change(self):
         assert (
             normalizeStringForPostscript("Sample copyright notice.")
@@ -227,7 +221,7 @@ class NormalizeStringForPostscriptTest(object):
         )
 
 
-class DateStringToTimeValueTest(object):
+class DateStringToTimeValueTest:
     def test_roundtrip_random_timestamp(self):
         timestamp = random.randint(0, 10 ** 9)
         ds = time.strftime("%Y/%m/%d %H:%M:%S", time.gmtime(timestamp))
