@@ -1,19 +1,18 @@
-from __future__ import print_function, division, absolute_import, unicode_literals
-import io
-from fontTools.misc.py23 import *
-from ufo2ft import (
-    compileOTF,
-    compileTTF,
-    compileInterpolatableTTFs,
-    compileVariableTTF,
-    compileVariableCFF2,
-)
-import warnings
 import difflib
+import io
 import os
 import sys
+
 import pytest
 
+from ufo2ft import (
+    compileInterpolatableTTFs,
+    compileOTF,
+    compileTTF,
+    compileVariableCFF2,
+    compileVariableTTF,
+)
+
 
 def getpath(filename):
     dirname = os.path.dirname(__file__)
@@ -39,12 +38,12 @@ def readLines(f):
 
 
 def expectTTX(font, expectedTTX, tables=None):
-    with open(getpath(expectedTTX), "r", encoding="utf-8") as f:
+    with open(getpath(expectedTTX), encoding="utf-8") as f:
         expected = readLines(f)
     font.recalcTimestamp = False
     font["head"].created, font["head"].modified = 3570196637, 3601822698
     font["head"].checkSumAdjustment = 0x12345678
-    f = UnicodeIO()
+    f = io.StringIO()
     font.saveXML(f, tables=tables)
 
     actual = readLines(f)
@@ -61,7 +60,7 @@ def useProductionNames(request):
     return request.param
 
 
-class IntegrationTest(object):
+class IntegrationTest:
 
     _layoutTables = ["GDEF", "GSUB", "GPOS", "BASE"]
 
@@ -119,6 +118,25 @@ class IntegrationTest(object):
         ttf = compileTTF(testufo, removeOverlaps=True, overlapsBackend="pathops")
         expectTTX(ttf, "TestFont-NoOverlaps-TTF-pathops.ttx")
 
+    def test_nestedComponents(self, FontClass):
+        ufo = FontClass(getpath("NestedComponents-Regular.ufo"))
+        ttf = compileTTF(ufo)
+        assert ttf["maxp"].maxComponentDepth != 1
+        ttf = compileTTF(ufo, flattenComponents=True)
+        assert ttf["maxp"].maxComponentDepth == 1
+
+    def test_nestedComponents_interpolatable(self, FontClass):
+        ufos = [
+            FontClass(getpath("NestedComponents-Regular.ufo")),
+            FontClass(getpath("NestedComponents-Bold.ufo")),
+        ]
+        ttfs = compileInterpolatableTTFs(ufos)
+        for ttf in ttfs:
+            assert ttf["maxp"].maxComponentDepth != 1
+        ttfs = compileInterpolatableTTFs(ufos, flattenComponents=True)
+        for ttf in ttfs:
+            assert ttf["maxp"].maxComponentDepth == 1
+
     def test_interpolatableTTFs_lazy(self, FontClass):
         # two same UFOs **must** be interpolatable
         ufos = [FontClass(getpath("TestFont.ufo")) for _ in range(2)]
@@ -126,17 +144,50 @@ class IntegrationTest(object):
         expectTTX(ttfs[0], "TestFont.ttx")
         expectTTX(ttfs[1], "TestFont.ttx")
 
-    def test_optimizeCFF_none(self, testufo):
-        otf = compileOTF(testufo, optimizeCFF=0)
-        expectTTX(otf, "TestFont-NoOptimize-CFF.ttx")
+    @pytest.mark.parametrize(
+        "cff_version, expected_ttx",
+        [(1, "TestFont-NoOptimize-CFF.ttx"), (2, "TestFont-NoOptimize-CFF2.ttx")],
+        ids=["cff1", "cff2"],
+    )
+    def test_optimizeCFF_none(self, testufo, cff_version, expected_ttx):
+        otf = compileOTF(testufo, optimizeCFF=0, cffVersion=cff_version)
+        expectTTX(otf, expected_ttx)
 
-    def test_optimizeCFF_specialize(self, testufo):
-        otf = compileOTF(testufo, optimizeCFF=1)
-        expectTTX(otf, "TestFont-Specialized-CFF.ttx")
+    @pytest.mark.parametrize(
+        "cff_version, expected_ttx",
+        [(1, "TestFont-Specialized-CFF.ttx"), (2, "TestFont-Specialized-CFF2.ttx")],
+        ids=["cff1", "cff2"],
+    )
+    def test_optimizeCFF_specialize(self, testufo, cff_version, expected_ttx):
+        otf = compileOTF(testufo, optimizeCFF=1, cffVersion=cff_version)
+        expectTTX(otf, expected_ttx)
 
-    def test_optimizeCFF_subroutinize(self, testufo):
-        otf = compileOTF(testufo, optimizeCFF=2)
-        expectTTX(otf, "TestFont-CFF.ttx")
+    @pytest.mark.parametrize(
+        "subroutinizer, cff_version, expected_ttx",
+        [
+            (None, 1, "TestFont-CFF.ttx"),
+            ("compreffor", 1, "TestFont-CFF.ttx"),
+            ("cffsubr", 1, "TestFont-CFF-cffsubr.ttx"),
+            (None, 2, "TestFont-CFF2-cffsubr.ttx"),
+            # ("compreffor", 2, "TestFont-CFF2-compreffor.ttx"),
+            ("cffsubr", 2, "TestFont-CFF2-cffsubr.ttx"),
+        ],
+        ids=[
+            "default-cff1",
+            "compreffor-cff1",
+            "cffsubr-cff1",
+            "default-cff2",
+            # "compreffor-cff2",
+            "cffsubr-cff2",
+        ],
+    )
+    def test_optimizeCFF_subroutinize(
+        self, testufo, cff_version, subroutinizer, expected_ttx
+    ):
+        otf = compileOTF(
+            testufo, optimizeCFF=2, cffVersion=cff_version, subroutinizer=subroutinizer
+        )
+        expectTTX(otf, expected_ttx)
 
     def test_compileVariableTTF(self, designspace, useProductionNames):
         varfont = compileVariableTTF(designspace, useProductionNames=useProductionNames)
@@ -158,14 +209,33 @@ class IntegrationTest(object):
             ),
         )
 
+    def test_compileVariableCFF2_subroutinized(self, designspace):
+        varfont = compileVariableCFF2(designspace, optimizeCFF=2)
+        expectTTX(varfont, "TestVariableFont-CFF2-cffsubr.ttx")
+
     def test_debugFeatureFile(self, designspace):
         tmp = io.StringIO()
 
-        varfont = compileVariableTTF(designspace, debugFeatureFile=tmp)
+        _ = compileVariableTTF(designspace, debugFeatureFile=tmp)
 
         assert "### LayerFont-Regular ###" in tmp.getvalue()
         assert "### LayerFont-Bold ###" in tmp.getvalue()
 
+    @pytest.mark.parametrize(
+        "output_format, options, expected_ttx",
+        [
+            ("TTF", {}, "TestFont-TTF-post3.ttx"),
+            ("OTF", {"cffVersion": 2}, "TestFont-CFF2-post3.ttx"),
+        ],
+    )
+    def test_drop_glyph_names(self, testufo, output_format, options, expected_ttx):
+        from ufo2ft.constants import KEEP_GLYPH_NAMES
+
+        testufo.lib[KEEP_GLYPH_NAMES] = False
+        compile_func = globals()[f"compile{output_format}"]
+        ttf = compile_func(testufo, **options)
+        expectTTX(ttf, expected_ttx)
+
 
 if __name__ == "__main__":
     sys.exit(pytest.main(sys.argv))
