@@ -1,22 +1,23 @@
-from __future__ import print_function, division, absolute_import, unicode_literals
-import os
 import logging
+import os
+
+import pytest
+from cu2qu.ufo import CURVE_TYPE_LIB_KEY
+from fontTools import designspaceLib
+
 import ufo2ft
-from ufo2ft.preProcessor import (
-    TTFPreProcessor,
-    TTFInterpolatablePreProcessor,
-    _init_explode_color_layer_glyphs_filter,
+from ufo2ft.constants import (
+    COLOR_LAYER_MAPPING_KEY,
+    COLOR_LAYERS_KEY,
+    COLOR_PALETTES_KEY,
 )
 from ufo2ft.filters import UFO2FT_FILTERS_KEY
 from ufo2ft.filters.explodeColorLayerGlyphs import ExplodeColorLayerGlyphsFilter
-from cu2qu.ufo import CURVE_TYPE_LIB_KEY
-from fontTools import designspaceLib
-from ufo2ft.constants import (
-    COLOR_LAYERS_KEY,
-    COLOR_LAYER_MAPPING_KEY,
-    COLOR_PALETTES_KEY,
+from ufo2ft.preProcessor import (
+    TTFInterpolatablePreProcessor,
+    TTFPreProcessor,
+    _init_explode_color_layer_glyphs_filter,
 )
-import pytest
 
 
 def getpath(filename):
@@ -30,7 +31,7 @@ def glyph_has_qcurve(ufo, glyph_name):
     )
 
 
-class TTFPreProcessorTest(object):
+class TTFPreProcessorTest:
     def test_no_inplace(self, FontClass):
         ufo = FontClass(getpath("TestFont.ufo"))
 
@@ -77,7 +78,7 @@ class TTFPreProcessorTest(object):
             assert glyph_has_qcurve(ufo, "c")
 
 
-class TTFInterpolatablePreProcessorTest(object):
+class TTFInterpolatablePreProcessorTest:
     def test_no_inplace(self, FontClass):
         ufo1 = FontClass(getpath("TestFont.ufo"))
         ufo2 = FontClass(getpath("TestFont.ufo"))
@@ -144,7 +145,7 @@ class TTFInterpolatablePreProcessorTest(object):
         assert (glyphSets[1]["a"][0][0].y - glyphSets[0]["a"][0][0].y) == 10
 
 
-class SkipExportGlyphsTest(object):
+class SkipExportGlyphsTest:
     def test_skip_export_glyphs_filter(self, FontClass):
         from ufo2ft.util import _GlyphSet
 
@@ -152,7 +153,7 @@ class SkipExportGlyphsTest(object):
         skipExportGlyphs = ["b", "d"]
         glyphSet = _GlyphSet.from_layer(ufo, skipExportGlyphs=skipExportGlyphs)
 
-        assert set(glyphSet.keys()) == set(["a", "c", "e", "f"])
+        assert set(glyphSet.keys()) == {"a", "c", "e", "f"}
         assert len(glyphSet["a"]) == 1
         assert not glyphSet["a"].components
         assert len(glyphSet["c"]) == 5  # 4 "d" components decomposed plus 1 outline
@@ -282,7 +283,7 @@ def color_ufo(FontClass):
     return ufo
 
 
-class InitExplodeColorLayerGlyphsFilterTest(object):
+class InitExplodeColorLayerGlyphsFilterTest:
     def test_no_color_palettes(self, FontClass):
         ufo = FontClass()
         filters = []
